'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

/* eslint-env browser */

var _require = require('redux'),
    createStore = _require.createStore,
    compose = _require.compose,
    applyMiddleware = _require.applyMiddleware,
    combineReducers = _require.combineReducers;

var createSagaMiddleware = require('redux-saga');

/**
 * 创建store
 *
 * 通过webpack替换获取到所有的reducer文件，再通过combineReducers组装到一起
 * 创建saga中间件，将中间件挂载到store上
 * 返回store
 *
 * @param    {object} state
 * @returns  {object} store
 *
 * @date     2018-03-01
 * @author   zhangyang<zhangyang@zhongan.com>
 */
module.exports = function () {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  var composeEnhancers = (typeof window === 'undefined' ? 'undefined' : _typeof(window)) === 'object' && window.__REDUX_DEVTOOLS_EXTENSION_COMPOSE__ ? window.__REDUX_DEVTOOLS_EXTENSION_COMPOSE__ : compose;
  var reducers = {};
  var commonReducers = void 0;
  var reducerFilePaths = global.__CLIENT_REDUCER__;
  var commonReducerFilePaths = global.__COMMON_CLIENT_REDUCER__;

  Object.keys(reducerFilePaths).forEach(function (i) {
    reducers[i] = require(global.__RELATIVE_PATH__ + '/' + reducerFilePaths[i]);
  });

  Object.keys(commonReducerFilePaths).forEach(function (i) {
    commonReducers = require(global.__COMMON_RELATIVE_PATH__ + '/' + commonReducerFilePaths[i]);
  });

  // Q: default ?
  var sagaMiddleware = createSagaMiddleware.default();

  var store = createStore(combineReducers({
    module: combineReducers(reducers),
    common: commonReducers
  }), state, composeEnhancers(applyMiddleware(sagaMiddleware)));

  store.runSaga = sagaMiddleware.run;
  store.close = function () {
    return store.dispatch(createSagaMiddleware.END);
  };

  return store;
};